import CGLineCap from './CoreGraphics/CGLineCap'
import CGLineJoin from './CoreGraphics/CGLineJoin'
import CGRect from './CoreGraphics/CGRect'
import CGSize from './CoreGraphics/CGSize'
import CGBlendMode from './CoreGraphics/CGBlendMode'
import CGPath from './CoreGraphics/CGPath'
import CGPoint from './CoreGraphics/CGPoint'
import CGPathApplierFunction from './CoreGraphics/CGPathApplierFunction'
import CGPathFillRule from './CoreGraphics/CGPathFillRule'
import CGMutablePath from './CoreGraphics/CGMutablePath'
import CGVector from './CoreGraphics/CGVector'
import _Buffer from './util/_Buffer'
import _HTMLCanvasElement from './util/_HTMLCanvasElement'
import _BinaryReader from './util/_BinaryReader'
import _File from './util/_File'
import _InstanceOf from './util/_InstanceOf'
import _TextReader from './util/_TextReader'
import _ClassList from './util/_ClassList'
import _AjaxRequest from './util/_AjaxRequest'
import CanUseWebGL2 from './util/CanUseWebGL2'
import _BinaryRequest from './util/_BinaryRequest'
import _FileReader from './util/_FileReader'
import MTLTessellationPartitionMode from './Metal/MTLTessellationPartitionMode'
import DispatchTimeInterval from './Dispatch/DispatchTimeInterval'
import DispatchObject from './Dispatch/DispatchObject'
import DispatchTime from './Dispatch/DispatchTime'
import DispatchQueue from './Dispatch/DispatchQueue'
import SCNBufferFrequency from './SceneKit/SCNBufferFrequency'
import SCNMatrix4Translate from './SceneKit/SCNMatrix4Translate'
import SCNActionRepeat from './SceneKit/SCNActionRepeat'
import SCNPhysicsVehicleWheel from './SceneKit/SCNPhysicsVehicleWheel'
import SCNPhysicsVehicle from './SceneKit/SCNPhysicsVehicle'
import SCNPhysicsHingeJoint from './SceneKit/SCNPhysicsHingeJoint'
import SCNGeometrySource from './SceneKit/SCNGeometrySource'
import SCNAnimationPlayer from './SceneKit/SCNAnimationPlayer'
import SCNTransformConstraint from './SceneKit/SCNTransformConstraint'
import SCNActionable from './SceneKit/SCNActionable'
import SCNLayer from './SceneKit/SCNLayer'
import SCNHitTestResult from './SceneKit/SCNHitTestResult'
import SCNParticleBirthLocation from './SceneKit/SCNParticleBirthLocation'
import SCNAudioSource from './SceneKit/SCNAudioSource'
import SCNRenderingAPI from './SceneKit/SCNRenderingAPI'
import SCNParticleModifierStage from './SceneKit/SCNParticleModifierStage'
import SCNMatrix4ToMat4 from './SceneKit/SCNMatrix4ToMat4'
import SCNShape from './SceneKit/SCNShape'
import SCNSceneExportDelegate from './SceneKit/SCNSceneExportDelegate'
import SCNParticleModifierBlock from './SceneKit/SCNParticleModifierBlock'
import SCNMatrix4Mult from './SceneKit/SCNMatrix4Mult'
import SCNFilterMode from './SceneKit/SCNFilterMode'
import SCNCamera from './SceneKit/SCNCamera'
import SCNFieldForceEvaluator from './SceneKit/SCNFieldForceEvaluator'
import _SCNDefaultVertexShader from './SceneKit/_SCNDefaultVertexShader'
import SCNTube from './SceneKit/SCNTube'
import SCNActionRunBlock from './SceneKit/SCNActionRunBlock'
import _SCNDefaultParticleFragmentShader from './SceneKit/_SCNDefaultParticleFragmentShader'
import SCNSceneSourceStatus from './SceneKit/SCNSceneSourceStatus'
import SCNFillMode from './SceneKit/SCNFillMode'
import SCNReferenceNode from './SceneKit/SCNReferenceNode'
import _SCNDefaultShadowFragmentShader from './SceneKit/_SCNDefaultShadowFragmentShader'
import SCNMatrix4MakeRotation from './SceneKit/SCNMatrix4MakeRotation'
import SCNBillboardConstraint from './SceneKit/SCNBillboardConstraint'
import SCNOrderedDictionary from './SceneKit/SCNOrderedDictionary'
import SCNActionHide from './SceneKit/SCNActionHide'
import _SCNObjLoader from './SceneKit/_SCNObjLoader'
import SCNActionMove from './SceneKit/SCNActionMove'
import SCNRenderer from './SceneKit/SCNRenderer'
import SCNConstraint from './SceneKit/SCNConstraint'
import SCNParticlePropertyController from './SceneKit/SCNParticlePropertyController'
import SCNCullMode from './SceneKit/SCNCullMode'
import SCNVector4Make from './SceneKit/SCNVector4Make'
import SCNTechniqueSupport from './SceneKit/SCNTechniqueSupport'
import SCNBufferStream from './SceneKit/SCNBufferStream'
import SCNActionPerformSelector from './SceneKit/SCNActionPerformSelector'
import SCNActionRotate from './SceneKit/SCNActionRotate'
import SCNPlane from './SceneKit/SCNPlane'
import SCNVector3FromFloat3 from './SceneKit/SCNVector3FromFloat3'
import SCNPhysicsShape from './SceneKit/SCNPhysicsShape'
import SCNDistanceConstraint from './SceneKit/SCNDistanceConstraint'
import SCNBufferBindingBlock from './SceneKit/SCNBufferBindingBlock'
import SCNVector3Zero from './SceneKit/SCNVector3Zero'
import SCNProgramDelegate from './SceneKit/SCNProgramDelegate'
import SCNChamferMode from './SceneKit/SCNChamferMode'
import SCNVector4 from './SceneKit/SCNVector4'
import SCNVector4ToFloat4 from './SceneKit/SCNVector4ToFloat4'
import SCNActionPlaySound from './SceneKit/SCNActionPlaySound'
import SCNSphere from './SceneKit/SCNSphere'
import SCNBindingBlock from './SceneKit/SCNBindingBlock'
import SCNActionFade from './SceneKit/SCNActionFade'
import SCNParticleInputMode from './SceneKit/SCNParticleInputMode'
import SCNAudioPlayer from './SceneKit/SCNAudioPlayer'
import SCNMatrix4MakeScale from './SceneKit/SCNMatrix4MakeScale'
import SCNMaterial from './SceneKit/SCNMaterial'
import SCNMatrix4Rotate from './SceneKit/SCNMatrix4Rotate'
import SCNQuaternion from './SceneKit/SCNQuaternion'
import _SCNDefaultHitTestVertexShader from './SceneKit/_SCNDefaultHitTestVertexShader'
import _SCNDefaultHitTestFragmentShader from './SceneKit/_SCNDefaultHitTestFragmentShader'
import SCNShaderModifierEntryPoint from './SceneKit/SCNShaderModifierEntryPoint'
import SCNPhysicsNoiseField from './SceneKit/SCNPhysicsNoiseField'
import SCNVector3FromGLKVector3 from './SceneKit/SCNVector3FromGLKVector3'
import SCNVector4FromGLKVector4 from './SceneKit/SCNVector4FromGLKVector4'
import SCNMatrix4ToGLKMatrix4 from './SceneKit/SCNMatrix4ToGLKMatrix4'
import SCNVector4ToGLKVector4 from './SceneKit/SCNVector4ToGLKVector4'
import SCNMatrix4 from './SceneKit/SCNMatrix4'
import SCNProgram from './SceneKit/SCNProgram'
import SCNAntialiasingMode from './SceneKit/SCNAntialiasingMode'
import SCNBoundingVolume from './SceneKit/SCNBoundingVolume'
import SCNTransparencyMode from './SceneKit/SCNTransparencyMode'
import SCNMatrix4EqualToMatrix4 from './SceneKit/SCNMatrix4EqualToMatrix4'
import SCNTessellationSmoothingMode from './SceneKit/SCNTessellationSmoothingMode'
import SCNPhysicsBehavior from './SceneKit/SCNPhysicsBehavior'
import SCNMatrix4Invert from './SceneKit/SCNMatrix4Invert'
import SCNParticleBirthDirection from './SceneKit/SCNParticleBirthDirection'
import SCNVector3Make from './SceneKit/SCNVector3Make'
import SCNPhysicsCollisionCategory from './SceneKit/SCNPhysicsCollisionCategory'
import SCNColorMask from './SceneKit/SCNColorMask'
import SCNParticleEvent from './SceneKit/SCNParticleEvent'
import _SCNDefaultFragmentShader from './SceneKit/_SCNDefaultFragmentShader'
import SCNActionReference from './SceneKit/SCNActionReference'
import SCNReferenceLoadingPolicy from './SceneKit/SCNReferenceLoadingPolicy'
import SCNActionSequence from './SceneKit/SCNActionSequence'
import SCNActionTimingMode from './SceneKit/SCNActionTimingMode'
import SCNBillboardAxis from './SceneKit/SCNBillboardAxis'
import SCNGeometryElement from './SceneKit/SCNGeometryElement'
import SCNShadable from './SceneKit/SCNShadable'
import SCNVector3ToGLKVector3 from './SceneKit/SCNVector3ToGLKVector3'
import SCNCylinder from './SceneKit/SCNCylinder'
import SCNLookAtConstraint from './SceneKit/SCNLookAtConstraint'
import SCNPhysicsBodyType from './SceneKit/SCNPhysicsBodyType'
import SCNTorus from './SceneKit/SCNTorus'
import SCNShadableHelper from './SceneKit/SCNShadableHelper'
import SCNLight from './SceneKit/SCNLight'
import SCNCameraProjectionDirection from './SceneKit/SCNCameraProjectionDirection'
import SCNSkinner from './SceneKit/SCNSkinner'
import SCNAnimatable from './SceneKit/SCNAnimatable'
import SCNGeometryTessellator from './SceneKit/SCNGeometryTessellator'
import SCNPhysicsContact from './SceneKit/SCNPhysicsContact'
import SCNAnimation from './SceneKit/SCNAnimation'
import SCNPhysicsContactDelegate from './SceneKit/SCNPhysicsContactDelegate'
import SCNMovabilityHint from './SceneKit/SCNMovabilityHint'
import SCNPhysicsFieldScope from './SceneKit/SCNPhysicsFieldScope'
import SCNActionScale from './SceneKit/SCNActionScale'
import SCNVector3 from './SceneKit/SCNVector3'
import SCNHitTestOption from './SceneKit/SCNHitTestOption'
import SCNActionCustom from './SceneKit/SCNActionCustom'
import SCNAnimationEvent from './SceneKit/SCNAnimationEvent'
import SCNIKConstraint from './SceneKit/SCNIKConstraint'
import SCNParticleSortingMode from './SceneKit/SCNParticleSortingMode'
import SCNMatrix4Scale from './SceneKit/SCNMatrix4Scale'
import SCNTechnique from './SceneKit/SCNTechnique'
import SCNParticleOrientationMode from './SceneKit/SCNParticleOrientationMode'
import SCNSceneSource from './SceneKit/SCNSceneSource'
import SCNNode from './SceneKit/SCNNode'
import SCNSceneExportProgressHandler from './SceneKit/SCNSceneExportProgressHandler'
import SCNPhysicsBallSocketJoint from './SceneKit/SCNPhysicsBallSocketJoint'
import SCNPhysicsBody from './SceneKit/SCNPhysicsBody'
import SCNDebugOptions from './SceneKit/SCNDebugOptions'
import SCNLevelOfDetail from './SceneKit/SCNLevelOfDetail'
import SCNParticleImageSequenceAnimationMode from './SceneKit/SCNParticleImageSequenceAnimationMode'
import SCNMatrix4FromMat4 from './SceneKit/SCNMatrix4FromMat4'
import SCNShadowMode from './SceneKit/SCNShadowMode'
import SCNVector4FromFloat4 from './SceneKit/SCNVector4FromFloat4'
import SCNMaterialProperty from './SceneKit/SCNMaterialProperty'
import SCNNodeRendererDelegate from './SceneKit/SCNNodeRendererDelegate'
import SCNPhysicsSliderJoint from './SceneKit/SCNPhysicsSliderJoint'
import SCNActionRunAction from './SceneKit/SCNActionRunAction'
import SCNBox from './SceneKit/SCNBox'
import SCNParticleEventBlock from './SceneKit/SCNParticleEventBlock'
import SCNTransaction from './SceneKit/SCNTransaction'
import SCNActionRemove from './SceneKit/SCNActionRemove'
import SCNBlendMode from './SceneKit/SCNBlendMode'
import SCNTimingFunction from './SceneKit/SCNTimingFunction'
import SCNCone from './SceneKit/SCNCone'
import SCNScene from './SceneKit/SCNScene'
import SCNPhysicsTurbulenceField from './SceneKit/SCNPhysicsTurbulenceField'
import SCNAction from './SceneKit/SCNAction'
import SCNFloor from './SceneKit/SCNFloor'
import SCNWrapMode from './SceneKit/SCNWrapMode'
import SCNText from './SceneKit/SCNText'
import _SCNDefaultPBRFragmentShader from './SceneKit/_SCNDefaultPBRFragmentShader'
import SCNActionWait from './SceneKit/SCNActionWait'
import SCNGeometry from './SceneKit/SCNGeometry'
import SCNParticleBlendMode from './SceneKit/SCNParticleBlendMode'
import SCNActionTimingFunction from './SceneKit/SCNActionTimingFunction'
import _SCNMtlLoader from './SceneKit/_SCNMtlLoader'
import SCNMorpherCalculationMode from './SceneKit/SCNMorpherCalculationMode'
import SCNVector3ToFloat3 from './SceneKit/SCNVector3ToFloat3'
import SCNGeometryPrimitiveType from './SceneKit/SCNGeometryPrimitiveType'
import SCNSceneSourceStatusHandler from './SceneKit/SCNSceneSourceStatusHandler'
import SCNPhysicsWorld from './SceneKit/SCNPhysicsWorld'
import SCNVector4EqualToVector4 from './SceneKit/SCNVector4EqualToVector4'
import SCNCapsule from './SceneKit/SCNCapsule'
import SCNSceneRenderer from './SceneKit/SCNSceneRenderer'
import SCNMatrix4MakeTranslation from './SceneKit/SCNMatrix4MakeTranslation'
import SCNAnimationEventBlock from './SceneKit/SCNAnimationEventBlock'
import SCNView from './SceneKit/SCNView'
import SCNMatrix4FromGLKMatrix4 from './SceneKit/SCNMatrix4FromGLKMatrix4'
import SCNMatrix4IsIdentity from './SceneKit/SCNMatrix4IsIdentity'
import SCNMorpher from './SceneKit/SCNMorpher'
import SCNSceneRendererDelegate from './SceneKit/SCNSceneRendererDelegate'
import SCNActionJavaScript from './SceneKit/SCNActionJavaScript'
import _SCNDefaultParticleVertexShader from './SceneKit/_SCNDefaultParticleVertexShader'
import SCNAccelerationConstraint from './SceneKit/SCNAccelerationConstraint'
import SCNPyramid from './SceneKit/SCNPyramid'
import _SCNDefaultShadowVertexShader from './SceneKit/_SCNDefaultShadowVertexShader'
import SCNActionGroup from './SceneKit/SCNActionGroup'
import SCNParticleSystem from './SceneKit/SCNParticleSystem'
import SCNVector3EqualToVector3 from './SceneKit/SCNVector3EqualToVector3'
import SCNPhysicsField from './SceneKit/SCNPhysicsField'
import NSColorSpace from './Foundation/NSColorSpace'
import NSCoder from './Foundation/NSCoder'
import NSMutableData from './Foundation/NSMutableData'
import NSMutableDictionary from './Foundation/NSMutableDictionary'
import NSData from './Foundation/NSData'
import NSURL from './Foundation/NSURL'
import NSKeyedArchiver from './Foundation/NSKeyedArchiver'
import NSValue from './Foundation/NSValue'
import NSArray from './Foundation/NSArray'
import NSNotification from './Foundation/NSNotification'
import NSKeyedUnarchiver from './Foundation/NSKeyedUnarchiver'
import NSDictionary from './Foundation/NSDictionary'
import NotificationCenter from './Foundation/NotificationCenter'
import NSMutableArray from './Foundation/NSMutableArray'
import AVAudioNode from './AVFoundation/AVAudioNode'
import AVAudioMixerNode from './AVFoundation/AVAudioMixerNode'
import GKAgent2D from './GameplayKit/GKAgent2D'
import GKSCNNodeComponent from './GameplayKit/GKSCNNodeComponent'
import GKGoal from './GameplayKit/GKGoal'
import GKPath from './GameplayKit/GKPath'
import GKBehavior from './GameplayKit/GKBehavior'
import GKScene from './GameplayKit/GKScene'
import GKComponent from './GameplayKit/GKComponent'
import GKAgentDelegate from './GameplayKit/GKAgentDelegate'
import GKEntity from './GameplayKit/GKEntity'
import GKAgent from './GameplayKit/GKAgent'
import SKSpriteNode from './SpriteKit/SKSpriteNode'
import SKColor from './SpriteKit/SKColor'
import SKAction from './SpriteKit/SKAction'
import SKActionTimingMode from './SpriteKit/SKActionTimingMode'
import SKBlendMode from './SpriteKit/SKBlendMode'
import SKScene from './SpriteKit/SKScene'
import SKFade from './SpriteKit/SKFade'
import SKLabelHorizontalAlignmentMode from './SpriteKit/SKLabelHorizontalAlignmentMode'
import SKSceneScaleMode from './SpriteKit/SKSceneScaleMode'
import SKNode from './SpriteKit/SKNode'
import SKGroup from './SpriteKit/SKGroup'
import SKEffectNode from './SpriteKit/SKEffectNode'
import SKTexture from './SpriteKit/SKTexture'
import SKTextureFilteringMode from './SpriteKit/SKTextureFilteringMode'
import SKShapeNode from './SpriteKit/SKShapeNode'
import SKSequence from './SpriteKit/SKSequence'
import SKRepeat from './SpriteKit/SKRepeat'
import SKScale from './SpriteKit/SKScale'
import SKLabelNode from './SpriteKit/SKLabelNode'
import SKLabelVerticalAlignmentMode from './SpriteKit/SKLabelVerticalAlignmentMode'
import SKWait from './SpriteKit/SKWait'
import GCExtendedGamepad from './GameController/GCExtendedGamepad'
import GCGamepad from './GameController/GCGamepad'
import GCControllerDirectionPad from './GameController/GCControllerDirectionPad'
import GCControllerElement from './GameController/GCControllerElement'
import GCMicroGamepad from './GameController/GCMicroGamepad'
import GCControllerAxisInput from './GameController/GCControllerAxisInput'
import GCControllerButtonInput from './GameController/GCControllerButtonInput'
import GCController from './GameController/GCController'
import CAMediaTiming from './QuartzCore/CAMediaTiming'
import CAAnimationGroup from './QuartzCore/CAAnimationGroup'
import CAMediaTimingFunction from './QuartzCore/CAMediaTimingFunction'
import CAPropertyAnimation from './QuartzCore/CAPropertyAnimation'
import CAKeyframeAnimation from './QuartzCore/CAKeyframeAnimation'
import CAAnimationDelegate from './QuartzCore/CAAnimationDelegate'
import CAAnimation from './QuartzCore/CAAnimation'
import CACodingProxy from './QuartzCore/CACodingProxy'
import CABasicAnimation from './QuartzCore/CABasicAnimation'
import LKNSArrayCodingProxy from './QuartzCore/LKNSArrayCodingProxy'
import CATransform3D from './QuartzCore/CATransform3D'
import CAAction from './QuartzCore/CAAction'
import LKNSValueCodingProxy from './QuartzCore/LKNSValueCodingProxy'
import CACurrentMediaTime from './QuartzCore/CACurrentMediaTime'
import NSColor from './AppKit/NSColor'
import NSColorSpaceModel from './AppKit/NSColorSpaceModel'
import CFAbsoluteTimeGetCurrent from './CoreFoundation/CFAbsoluteTimeGetCurrent'
import NSObject from './ObjectiveC/NSObject'
import * as constants from './constants'

_ClassList.registerClass(CGLineCap, 'CGLineCap')
_ClassList.registerClass(CGLineJoin, 'CGLineJoin')
_ClassList.registerClass(CGRect, 'CGRect')
_ClassList.registerClass(CGSize, 'CGSize')
_ClassList.registerClass(CGBlendMode, 'CGBlendMode')
_ClassList.registerClass(CGPath, 'CGPath')
_ClassList.registerClass(CGPoint, 'CGPoint')
_ClassList.registerClass(CGPathApplierFunction, 'CGPathApplierFunction')
_ClassList.registerClass(CGPathFillRule, 'CGPathFillRule')
_ClassList.registerClass(CGMutablePath, 'CGMutablePath')
_ClassList.registerClass(CGVector, 'CGVector')
_ClassList.registerClass(MTLTessellationPartitionMode, 'MTLTessellationPartitionMode')
_ClassList.registerClass(DispatchTimeInterval, 'DispatchTimeInterval')
_ClassList.registerClass(DispatchObject, 'DispatchObject')
_ClassList.registerClass(DispatchTime, 'DispatchTime')
_ClassList.registerClass(DispatchQueue, 'DispatchQueue')
_ClassList.registerClass(SCNBufferFrequency, 'SCNBufferFrequency')
_ClassList.registerClass(SCNMatrix4Translate, 'SCNMatrix4Translate')
_ClassList.registerClass(SCNActionRepeat, 'SCNActionRepeat')
_ClassList.registerClass(SCNPhysicsVehicleWheel, 'SCNPhysicsVehicleWheel')
_ClassList.registerClass(SCNPhysicsVehicle, 'SCNPhysicsVehicle')
_ClassList.registerClass(SCNPhysicsHingeJoint, 'SCNPhysicsHingeJoint')
_ClassList.registerClass(SCNGeometrySource, 'SCNGeometrySource')
_ClassList.registerClass(SCNAnimationPlayer, 'SCNAnimationPlayer')
_ClassList.registerClass(SCNTransformConstraint, 'SCNTransformConstraint')
_ClassList.registerClass(SCNActionable, 'SCNActionable')
_ClassList.registerClass(SCNLayer, 'SCNLayer')
_ClassList.registerClass(SCNHitTestResult, 'SCNHitTestResult')
_ClassList.registerClass(SCNParticleBirthLocation, 'SCNParticleBirthLocation')
_ClassList.registerClass(SCNAudioSource, 'SCNAudioSource')
_ClassList.registerClass(SCNRenderingAPI, 'SCNRenderingAPI')
_ClassList.registerClass(SCNParticleModifierStage, 'SCNParticleModifierStage')
_ClassList.registerClass(SCNMatrix4ToMat4, 'SCNMatrix4ToMat4')
_ClassList.registerClass(SCNShape, 'SCNShape')
_ClassList.registerClass(SCNSceneExportDelegate, 'SCNSceneExportDelegate')
_ClassList.registerClass(SCNParticleModifierBlock, 'SCNParticleModifierBlock')
_ClassList.registerClass(SCNMatrix4Mult, 'SCNMatrix4Mult')
_ClassList.registerClass(SCNFilterMode, 'SCNFilterMode')
_ClassList.registerClass(SCNCamera, 'SCNCamera')
_ClassList.registerClass(SCNFieldForceEvaluator, 'SCNFieldForceEvaluator')
_ClassList.registerClass(_SCNDefaultVertexShader, '_SCNDefaultVertexShader')
_ClassList.registerClass(SCNTube, 'SCNTube')
_ClassList.registerClass(SCNActionRunBlock, 'SCNActionRunBlock')
_ClassList.registerClass(_SCNDefaultParticleFragmentShader, '_SCNDefaultParticleFragmentShader')
_ClassList.registerClass(SCNSceneSourceStatus, 'SCNSceneSourceStatus')
_ClassList.registerClass(SCNFillMode, 'SCNFillMode')
_ClassList.registerClass(SCNReferenceNode, 'SCNReferenceNode')
_ClassList.registerClass(_SCNDefaultShadowFragmentShader, '_SCNDefaultShadowFragmentShader')
_ClassList.registerClass(SCNMatrix4MakeRotation, 'SCNMatrix4MakeRotation')
_ClassList.registerClass(SCNBillboardConstraint, 'SCNBillboardConstraint')
_ClassList.registerClass(SCNOrderedDictionary, 'SCNOrderedDictionary')
_ClassList.registerClass(SCNActionHide, 'SCNActionHide')
_ClassList.registerClass(_SCNObjLoader, '_SCNObjLoader')
_ClassList.registerClass(SCNActionMove, 'SCNActionMove')
_ClassList.registerClass(SCNRenderer, 'SCNRenderer')
_ClassList.registerClass(SCNConstraint, 'SCNConstraint')
_ClassList.registerClass(SCNParticlePropertyController, 'SCNParticlePropertyController')
_ClassList.registerClass(SCNCullMode, 'SCNCullMode')
_ClassList.registerClass(SCNVector4Make, 'SCNVector4Make')
_ClassList.registerClass(SCNTechniqueSupport, 'SCNTechniqueSupport')
_ClassList.registerClass(SCNBufferStream, 'SCNBufferStream')
_ClassList.registerClass(SCNActionPerformSelector, 'SCNActionPerformSelector')
_ClassList.registerClass(SCNActionRotate, 'SCNActionRotate')
_ClassList.registerClass(SCNPlane, 'SCNPlane')
_ClassList.registerClass(SCNVector3FromFloat3, 'SCNVector3FromFloat3')
_ClassList.registerClass(SCNPhysicsShape, 'SCNPhysicsShape')
_ClassList.registerClass(SCNDistanceConstraint, 'SCNDistanceConstraint')
_ClassList.registerClass(SCNBufferBindingBlock, 'SCNBufferBindingBlock')
_ClassList.registerClass(SCNVector3Zero, 'SCNVector3Zero')
_ClassList.registerClass(SCNProgramDelegate, 'SCNProgramDelegate')
_ClassList.registerClass(SCNChamferMode, 'SCNChamferMode')
_ClassList.registerClass(SCNVector4, 'SCNVector4')
_ClassList.registerClass(SCNVector4ToFloat4, 'SCNVector4ToFloat4')
_ClassList.registerClass(SCNActionPlaySound, 'SCNActionPlaySound')
_ClassList.registerClass(SCNSphere, 'SCNSphere')
_ClassList.registerClass(SCNBindingBlock, 'SCNBindingBlock')
_ClassList.registerClass(SCNActionFade, 'SCNActionFade')
_ClassList.registerClass(SCNParticleInputMode, 'SCNParticleInputMode')
_ClassList.registerClass(SCNAudioPlayer, 'SCNAudioPlayer')
_ClassList.registerClass(SCNMatrix4MakeScale, 'SCNMatrix4MakeScale')
_ClassList.registerClass(SCNMaterial, 'SCNMaterial')
_ClassList.registerClass(SCNMatrix4Rotate, 'SCNMatrix4Rotate')
_ClassList.registerClass(SCNQuaternion, 'SCNQuaternion')
_ClassList.registerClass(_SCNDefaultHitTestVertexShader, '_SCNDefaultHitTestVertexShader')
_ClassList.registerClass(_SCNDefaultHitTestFragmentShader, '_SCNDefaultHitTestFragmentShader')
_ClassList.registerClass(SCNShaderModifierEntryPoint, 'SCNShaderModifierEntryPoint')
_ClassList.registerClass(SCNPhysicsNoiseField, 'SCNPhysicsNoiseField')
_ClassList.registerClass(SCNVector3FromGLKVector3, 'SCNVector3FromGLKVector3')
_ClassList.registerClass(SCNVector4FromGLKVector4, 'SCNVector4FromGLKVector4')
_ClassList.registerClass(SCNMatrix4ToGLKMatrix4, 'SCNMatrix4ToGLKMatrix4')
_ClassList.registerClass(SCNVector4ToGLKVector4, 'SCNVector4ToGLKVector4')
_ClassList.registerClass(SCNMatrix4, 'SCNMatrix4')
_ClassList.registerClass(SCNProgram, 'SCNProgram')
_ClassList.registerClass(SCNAntialiasingMode, 'SCNAntialiasingMode')
_ClassList.registerClass(SCNBoundingVolume, 'SCNBoundingVolume')
_ClassList.registerClass(SCNTransparencyMode, 'SCNTransparencyMode')
_ClassList.registerClass(SCNMatrix4EqualToMatrix4, 'SCNMatrix4EqualToMatrix4')
_ClassList.registerClass(SCNTessellationSmoothingMode, 'SCNTessellationSmoothingMode')
_ClassList.registerClass(SCNPhysicsBehavior, 'SCNPhysicsBehavior')
_ClassList.registerClass(SCNMatrix4Invert, 'SCNMatrix4Invert')
_ClassList.registerClass(SCNParticleBirthDirection, 'SCNParticleBirthDirection')
_ClassList.registerClass(SCNVector3Make, 'SCNVector3Make')
_ClassList.registerClass(SCNPhysicsCollisionCategory, 'SCNPhysicsCollisionCategory')
_ClassList.registerClass(SCNColorMask, 'SCNColorMask')
_ClassList.registerClass(SCNParticleEvent, 'SCNParticleEvent')
_ClassList.registerClass(_SCNDefaultFragmentShader, '_SCNDefaultFragmentShader')
_ClassList.registerClass(SCNActionReference, 'SCNActionReference')
_ClassList.registerClass(SCNReferenceLoadingPolicy, 'SCNReferenceLoadingPolicy')
_ClassList.registerClass(SCNActionSequence, 'SCNActionSequence')
_ClassList.registerClass(SCNActionTimingMode, 'SCNActionTimingMode')
_ClassList.registerClass(SCNBillboardAxis, 'SCNBillboardAxis')
_ClassList.registerClass(SCNGeometryElement, 'SCNGeometryElement')
_ClassList.registerClass(SCNShadable, 'SCNShadable')
_ClassList.registerClass(SCNVector3ToGLKVector3, 'SCNVector3ToGLKVector3')
_ClassList.registerClass(SCNCylinder, 'SCNCylinder')
_ClassList.registerClass(SCNLookAtConstraint, 'SCNLookAtConstraint')
_ClassList.registerClass(SCNPhysicsBodyType, 'SCNPhysicsBodyType')
_ClassList.registerClass(SCNTorus, 'SCNTorus')
_ClassList.registerClass(SCNShadableHelper, 'SCNShadableHelper')
_ClassList.registerClass(SCNLight, 'SCNLight')
_ClassList.registerClass(SCNCameraProjectionDirection, 'SCNCameraProjectionDirection')
_ClassList.registerClass(SCNSkinner, 'SCNSkinner')
_ClassList.registerClass(SCNAnimatable, 'SCNAnimatable')
_ClassList.registerClass(SCNGeometryTessellator, 'SCNGeometryTessellator')
_ClassList.registerClass(SCNPhysicsContact, 'SCNPhysicsContact')
_ClassList.registerClass(SCNAnimation, 'SCNAnimation')
_ClassList.registerClass(SCNPhysicsContactDelegate, 'SCNPhysicsContactDelegate')
_ClassList.registerClass(SCNMovabilityHint, 'SCNMovabilityHint')
_ClassList.registerClass(SCNPhysicsFieldScope, 'SCNPhysicsFieldScope')
_ClassList.registerClass(SCNActionScale, 'SCNActionScale')
_ClassList.registerClass(SCNVector3, 'SCNVector3')
_ClassList.registerClass(SCNHitTestOption, 'SCNHitTestOption')
_ClassList.registerClass(SCNActionCustom, 'SCNActionCustom')
_ClassList.registerClass(SCNAnimationEvent, 'SCNAnimationEvent')
_ClassList.registerClass(SCNIKConstraint, 'SCNIKConstraint')
_ClassList.registerClass(SCNParticleSortingMode, 'SCNParticleSortingMode')
_ClassList.registerClass(SCNMatrix4Scale, 'SCNMatrix4Scale')
_ClassList.registerClass(SCNTechnique, 'SCNTechnique')
_ClassList.registerClass(SCNParticleOrientationMode, 'SCNParticleOrientationMode')
_ClassList.registerClass(SCNSceneSource, 'SCNSceneSource')
_ClassList.registerClass(SCNNode, 'SCNNode')
_ClassList.registerClass(SCNSceneExportProgressHandler, 'SCNSceneExportProgressHandler')
_ClassList.registerClass(SCNPhysicsBallSocketJoint, 'SCNPhysicsBallSocketJoint')
_ClassList.registerClass(SCNPhysicsBody, 'SCNPhysicsBody')
_ClassList.registerClass(SCNDebugOptions, 'SCNDebugOptions')
_ClassList.registerClass(SCNLevelOfDetail, 'SCNLevelOfDetail')
_ClassList.registerClass(SCNParticleImageSequenceAnimationMode, 'SCNParticleImageSequenceAnimationMode')
_ClassList.registerClass(SCNMatrix4FromMat4, 'SCNMatrix4FromMat4')
_ClassList.registerClass(SCNShadowMode, 'SCNShadowMode')
_ClassList.registerClass(SCNVector4FromFloat4, 'SCNVector4FromFloat4')
_ClassList.registerClass(SCNMaterialProperty, 'SCNMaterialProperty')
_ClassList.registerClass(SCNNodeRendererDelegate, 'SCNNodeRendererDelegate')
_ClassList.registerClass(SCNPhysicsSliderJoint, 'SCNPhysicsSliderJoint')
_ClassList.registerClass(SCNActionRunAction, 'SCNActionRunAction')
_ClassList.registerClass(SCNBox, 'SCNBox')
_ClassList.registerClass(SCNParticleEventBlock, 'SCNParticleEventBlock')
_ClassList.registerClass(SCNTransaction, 'SCNTransaction')
_ClassList.registerClass(SCNActionRemove, 'SCNActionRemove')
_ClassList.registerClass(SCNBlendMode, 'SCNBlendMode')
_ClassList.registerClass(SCNTimingFunction, 'SCNTimingFunction')
_ClassList.registerClass(SCNCone, 'SCNCone')
_ClassList.registerClass(SCNScene, 'SCNScene')
_ClassList.registerClass(SCNPhysicsTurbulenceField, 'SCNPhysicsTurbulenceField')
_ClassList.registerClass(SCNAction, 'SCNAction')
_ClassList.registerClass(SCNFloor, 'SCNFloor')
_ClassList.registerClass(SCNWrapMode, 'SCNWrapMode')
_ClassList.registerClass(SCNText, 'SCNText')
_ClassList.registerClass(_SCNDefaultPBRFragmentShader, '_SCNDefaultPBRFragmentShader')
_ClassList.registerClass(SCNActionWait, 'SCNActionWait')
_ClassList.registerClass(SCNGeometry, 'SCNGeometry')
_ClassList.registerClass(SCNParticleBlendMode, 'SCNParticleBlendMode')
_ClassList.registerClass(SCNActionTimingFunction, 'SCNActionTimingFunction')
_ClassList.registerClass(_SCNMtlLoader, '_SCNMtlLoader')
_ClassList.registerClass(SCNMorpherCalculationMode, 'SCNMorpherCalculationMode')
_ClassList.registerClass(SCNVector3ToFloat3, 'SCNVector3ToFloat3')
_ClassList.registerClass(SCNGeometryPrimitiveType, 'SCNGeometryPrimitiveType')
_ClassList.registerClass(SCNSceneSourceStatusHandler, 'SCNSceneSourceStatusHandler')
_ClassList.registerClass(SCNPhysicsWorld, 'SCNPhysicsWorld')
_ClassList.registerClass(SCNVector4EqualToVector4, 'SCNVector4EqualToVector4')
_ClassList.registerClass(SCNCapsule, 'SCNCapsule')
_ClassList.registerClass(SCNSceneRenderer, 'SCNSceneRenderer')
_ClassList.registerClass(SCNMatrix4MakeTranslation, 'SCNMatrix4MakeTranslation')
_ClassList.registerClass(SCNAnimationEventBlock, 'SCNAnimationEventBlock')
_ClassList.registerClass(SCNView, 'SCNView')
_ClassList.registerClass(SCNMatrix4FromGLKMatrix4, 'SCNMatrix4FromGLKMatrix4')
_ClassList.registerClass(SCNMatrix4IsIdentity, 'SCNMatrix4IsIdentity')
_ClassList.registerClass(SCNMorpher, 'SCNMorpher')
_ClassList.registerClass(SCNSceneRendererDelegate, 'SCNSceneRendererDelegate')
_ClassList.registerClass(SCNActionJavaScript, 'SCNActionJavaScript')
_ClassList.registerClass(_SCNDefaultParticleVertexShader, '_SCNDefaultParticleVertexShader')
_ClassList.registerClass(SCNAccelerationConstraint, 'SCNAccelerationConstraint')
_ClassList.registerClass(SCNPyramid, 'SCNPyramid')
_ClassList.registerClass(_SCNDefaultShadowVertexShader, '_SCNDefaultShadowVertexShader')
_ClassList.registerClass(SCNActionGroup, 'SCNActionGroup')
_ClassList.registerClass(SCNParticleSystem, 'SCNParticleSystem')
_ClassList.registerClass(SCNVector3EqualToVector3, 'SCNVector3EqualToVector3')
_ClassList.registerClass(SCNPhysicsField, 'SCNPhysicsField')
_ClassList.registerClass(NSColorSpace, 'NSColorSpace')
_ClassList.registerClass(NSCoder, 'NSCoder')
_ClassList.registerClass(NSMutableData, 'NSMutableData')
_ClassList.registerClass(NSMutableDictionary, 'NSMutableDictionary')
_ClassList.registerClass(NSData, 'NSData')
_ClassList.registerClass(NSURL, 'NSURL')
_ClassList.registerClass(NSKeyedArchiver, 'NSKeyedArchiver')
_ClassList.registerClass(NSValue, 'NSValue')
_ClassList.registerClass(NSArray, 'NSArray')
_ClassList.registerClass(NSNotification, 'NSNotification')
_ClassList.registerClass(NSKeyedUnarchiver, 'NSKeyedUnarchiver')
_ClassList.registerClass(NSDictionary, 'NSDictionary')
_ClassList.registerClass(NotificationCenter, 'NotificationCenter')
_ClassList.registerClass(NSMutableArray, 'NSMutableArray')
_ClassList.registerClass(AVAudioNode, 'AVAudioNode')
_ClassList.registerClass(AVAudioMixerNode, 'AVAudioMixerNode')
_ClassList.registerClass(GKAgent2D, 'GKAgent2D')
_ClassList.registerClass(GKSCNNodeComponent, 'GKSCNNodeComponent')
_ClassList.registerClass(GKGoal, 'GKGoal')
_ClassList.registerClass(GKPath, 'GKPath')
_ClassList.registerClass(GKBehavior, 'GKBehavior')
_ClassList.registerClass(GKScene, 'GKScene')
_ClassList.registerClass(GKComponent, 'GKComponent')
_ClassList.registerClass(GKAgentDelegate, 'GKAgentDelegate')
_ClassList.registerClass(GKEntity, 'GKEntity')
_ClassList.registerClass(GKAgent, 'GKAgent')
_ClassList.registerClass(SKSpriteNode, 'SKSpriteNode')
_ClassList.registerClass(SKColor, 'SKColor')
_ClassList.registerClass(SKAction, 'SKAction')
_ClassList.registerClass(SKActionTimingMode, 'SKActionTimingMode')
_ClassList.registerClass(SKBlendMode, 'SKBlendMode')
_ClassList.registerClass(SKScene, 'SKScene')
_ClassList.registerClass(SKFade, 'SKFade')
_ClassList.registerClass(SKLabelHorizontalAlignmentMode, 'SKLabelHorizontalAlignmentMode')
_ClassList.registerClass(SKSceneScaleMode, 'SKSceneScaleMode')
_ClassList.registerClass(SKNode, 'SKNode')
_ClassList.registerClass(SKGroup, 'SKGroup')
_ClassList.registerClass(SKEffectNode, 'SKEffectNode')
_ClassList.registerClass(SKTexture, 'SKTexture')
_ClassList.registerClass(SKTextureFilteringMode, 'SKTextureFilteringMode')
_ClassList.registerClass(SKShapeNode, 'SKShapeNode')
_ClassList.registerClass(SKSequence, 'SKSequence')
_ClassList.registerClass(SKRepeat, 'SKRepeat')
_ClassList.registerClass(SKScale, 'SKScale')
_ClassList.registerClass(SKLabelNode, 'SKLabelNode')
_ClassList.registerClass(SKLabelVerticalAlignmentMode, 'SKLabelVerticalAlignmentMode')
_ClassList.registerClass(SKWait, 'SKWait')
_ClassList.registerClass(GCExtendedGamepad, 'GCExtendedGamepad')
_ClassList.registerClass(GCGamepad, 'GCGamepad')
_ClassList.registerClass(GCControllerDirectionPad, 'GCControllerDirectionPad')
_ClassList.registerClass(GCControllerElement, 'GCControllerElement')
_ClassList.registerClass(GCMicroGamepad, 'GCMicroGamepad')
_ClassList.registerClass(GCControllerAxisInput, 'GCControllerAxisInput')
_ClassList.registerClass(GCControllerButtonInput, 'GCControllerButtonInput')
_ClassList.registerClass(GCController, 'GCController')
_ClassList.registerClass(CAMediaTiming, 'CAMediaTiming')
_ClassList.registerClass(CAAnimationGroup, 'CAAnimationGroup')
_ClassList.registerClass(CAMediaTimingFunction, 'CAMediaTimingFunction')
_ClassList.registerClass(CAPropertyAnimation, 'CAPropertyAnimation')
_ClassList.registerClass(CAKeyframeAnimation, 'CAKeyframeAnimation')
_ClassList.registerClass(CAAnimationDelegate, 'CAAnimationDelegate')
_ClassList.registerClass(CAAnimation, 'CAAnimation')
_ClassList.registerClass(CACodingProxy, 'CACodingProxy')
_ClassList.registerClass(CABasicAnimation, 'CABasicAnimation')
_ClassList.registerClass(LKNSArrayCodingProxy, 'LKNSArrayCodingProxy')
_ClassList.registerClass(CATransform3D, 'CATransform3D')
_ClassList.registerClass(CAAction, 'CAAction')
_ClassList.registerClass(LKNSValueCodingProxy, 'LKNSValueCodingProxy')
_ClassList.registerClass(CACurrentMediaTime, 'CACurrentMediaTime')
_ClassList.registerClass(NSColor, 'NSColor')
_ClassList.registerClass(NSColorSpaceModel, 'NSColorSpaceModel')
_ClassList.registerClass(CFAbsoluteTimeGetCurrent, 'CFAbsoluteTimeGetCurrent')
_ClassList.registerClass(NSObject, 'NSObject')

/*global exports*/
exports.CGLineCap = CGLineCap
exports.CGLineJoin = CGLineJoin
exports.CGRect = CGRect
exports.CGSize = CGSize
exports.CGBlendMode = CGBlendMode
exports.CGPath = CGPath
exports.CGPoint = CGPoint
exports.CGPathApplierFunction = CGPathApplierFunction
exports.CGPathFillRule = CGPathFillRule
exports.CGMutablePath = CGMutablePath
exports.CGVector = CGVector
exports._Buffer = _Buffer
exports._HTMLCanvasElement = _HTMLCanvasElement
exports._BinaryReader = _BinaryReader
exports._File = _File
exports._InstanceOf = _InstanceOf
exports._TextReader = _TextReader
exports._ClassList = _ClassList
exports._AjaxRequest = _AjaxRequest
exports.CanUseWebGL2 = CanUseWebGL2
exports._BinaryRequest = _BinaryRequest
exports._FileReader = _FileReader
exports.MTLTessellationPartitionMode = MTLTessellationPartitionMode
exports.DispatchTimeInterval = DispatchTimeInterval
exports.DispatchObject = DispatchObject
exports.DispatchTime = DispatchTime
exports.DispatchQueue = DispatchQueue
exports.SCNBufferFrequency = SCNBufferFrequency
exports.SCNMatrix4Translate = SCNMatrix4Translate
exports.SCNActionRepeat = SCNActionRepeat
exports.SCNPhysicsVehicleWheel = SCNPhysicsVehicleWheel
exports.SCNPhysicsVehicle = SCNPhysicsVehicle
exports.SCNPhysicsHingeJoint = SCNPhysicsHingeJoint
exports.SCNGeometrySource = SCNGeometrySource
exports.SCNAnimationPlayer = SCNAnimationPlayer
exports.SCNTransformConstraint = SCNTransformConstraint
exports.SCNActionable = SCNActionable
exports.SCNLayer = SCNLayer
exports.SCNHitTestResult = SCNHitTestResult
exports.SCNParticleBirthLocation = SCNParticleBirthLocation
exports.SCNAudioSource = SCNAudioSource
exports.SCNRenderingAPI = SCNRenderingAPI
exports.SCNParticleModifierStage = SCNParticleModifierStage
exports.SCNMatrix4ToMat4 = SCNMatrix4ToMat4
exports.SCNShape = SCNShape
exports.SCNSceneExportDelegate = SCNSceneExportDelegate
exports.SCNParticleModifierBlock = SCNParticleModifierBlock
exports.SCNMatrix4Mult = SCNMatrix4Mult
exports.SCNFilterMode = SCNFilterMode
exports.SCNCamera = SCNCamera
exports.SCNFieldForceEvaluator = SCNFieldForceEvaluator
exports._SCNDefaultVertexShader = _SCNDefaultVertexShader
exports.SCNTube = SCNTube
exports.SCNActionRunBlock = SCNActionRunBlock
exports._SCNDefaultParticleFragmentShader = _SCNDefaultParticleFragmentShader
exports.SCNSceneSourceStatus = SCNSceneSourceStatus
exports.SCNFillMode = SCNFillMode
exports.SCNReferenceNode = SCNReferenceNode
exports._SCNDefaultShadowFragmentShader = _SCNDefaultShadowFragmentShader
exports.SCNMatrix4MakeRotation = SCNMatrix4MakeRotation
exports.SCNBillboardConstraint = SCNBillboardConstraint
exports.SCNOrderedDictionary = SCNOrderedDictionary
exports.SCNActionHide = SCNActionHide
exports._SCNObjLoader = _SCNObjLoader
exports.SCNActionMove = SCNActionMove
exports.SCNRenderer = SCNRenderer
exports.SCNConstraint = SCNConstraint
exports.SCNParticlePropertyController = SCNParticlePropertyController
exports.SCNCullMode = SCNCullMode
exports.SCNVector4Make = SCNVector4Make
exports.SCNTechniqueSupport = SCNTechniqueSupport
exports.SCNBufferStream = SCNBufferStream
exports.SCNActionPerformSelector = SCNActionPerformSelector
exports.SCNActionRotate = SCNActionRotate
exports.SCNPlane = SCNPlane
exports.SCNVector3FromFloat3 = SCNVector3FromFloat3
exports.SCNPhysicsShape = SCNPhysicsShape
exports.SCNDistanceConstraint = SCNDistanceConstraint
exports.SCNBufferBindingBlock = SCNBufferBindingBlock
exports.SCNVector3Zero = SCNVector3Zero
exports.SCNProgramDelegate = SCNProgramDelegate
exports.SCNChamferMode = SCNChamferMode
exports.SCNVector4 = SCNVector4
exports.SCNVector4ToFloat4 = SCNVector4ToFloat4
exports.SCNActionPlaySound = SCNActionPlaySound
exports.SCNSphere = SCNSphere
exports.SCNBindingBlock = SCNBindingBlock
exports.SCNActionFade = SCNActionFade
exports.SCNParticleInputMode = SCNParticleInputMode
exports.SCNAudioPlayer = SCNAudioPlayer
exports.SCNMatrix4MakeScale = SCNMatrix4MakeScale
exports.SCNMaterial = SCNMaterial
exports.SCNMatrix4Rotate = SCNMatrix4Rotate
exports.SCNQuaternion = SCNQuaternion
exports._SCNDefaultHitTestVertexShader = _SCNDefaultHitTestVertexShader
exports._SCNDefaultHitTestFragmentShader = _SCNDefaultHitTestFragmentShader
exports.SCNShaderModifierEntryPoint = SCNShaderModifierEntryPoint
exports.SCNPhysicsNoiseField = SCNPhysicsNoiseField
exports.SCNVector3FromGLKVector3 = SCNVector3FromGLKVector3
exports.SCNVector4FromGLKVector4 = SCNVector4FromGLKVector4
exports.SCNMatrix4ToGLKMatrix4 = SCNMatrix4ToGLKMatrix4
exports.SCNVector4ToGLKVector4 = SCNVector4ToGLKVector4
exports.SCNMatrix4 = SCNMatrix4
exports.SCNProgram = SCNProgram
exports.SCNAntialiasingMode = SCNAntialiasingMode
exports.SCNBoundingVolume = SCNBoundingVolume
exports.SCNTransparencyMode = SCNTransparencyMode
exports.SCNMatrix4EqualToMatrix4 = SCNMatrix4EqualToMatrix4
exports.SCNTessellationSmoothingMode = SCNTessellationSmoothingMode
exports.SCNPhysicsBehavior = SCNPhysicsBehavior
exports.SCNMatrix4Invert = SCNMatrix4Invert
exports.SCNParticleBirthDirection = SCNParticleBirthDirection
exports.SCNVector3Make = SCNVector3Make
exports.SCNPhysicsCollisionCategory = SCNPhysicsCollisionCategory
exports.SCNColorMask = SCNColorMask
exports.SCNParticleEvent = SCNParticleEvent
exports._SCNDefaultFragmentShader = _SCNDefaultFragmentShader
exports.SCNActionReference = SCNActionReference
exports.SCNReferenceLoadingPolicy = SCNReferenceLoadingPolicy
exports.SCNActionSequence = SCNActionSequence
exports.SCNActionTimingMode = SCNActionTimingMode
exports.SCNBillboardAxis = SCNBillboardAxis
exports.SCNGeometryElement = SCNGeometryElement
exports.SCNShadable = SCNShadable
exports.SCNVector3ToGLKVector3 = SCNVector3ToGLKVector3
exports.SCNCylinder = SCNCylinder
exports.SCNLookAtConstraint = SCNLookAtConstraint
exports.SCNPhysicsBodyType = SCNPhysicsBodyType
exports.SCNTorus = SCNTorus
exports.SCNShadableHelper = SCNShadableHelper
exports.SCNLight = SCNLight
exports.SCNCameraProjectionDirection = SCNCameraProjectionDirection
exports.SCNSkinner = SCNSkinner
exports.SCNAnimatable = SCNAnimatable
exports.SCNGeometryTessellator = SCNGeometryTessellator
exports.SCNPhysicsContact = SCNPhysicsContact
exports.SCNAnimation = SCNAnimation
exports.SCNPhysicsContactDelegate = SCNPhysicsContactDelegate
exports.SCNMovabilityHint = SCNMovabilityHint
exports.SCNPhysicsFieldScope = SCNPhysicsFieldScope
exports.SCNActionScale = SCNActionScale
exports.SCNVector3 = SCNVector3
exports.SCNHitTestOption = SCNHitTestOption
exports.SCNActionCustom = SCNActionCustom
exports.SCNAnimationEvent = SCNAnimationEvent
exports.SCNIKConstraint = SCNIKConstraint
exports.SCNParticleSortingMode = SCNParticleSortingMode
exports.SCNMatrix4Scale = SCNMatrix4Scale
exports.SCNTechnique = SCNTechnique
exports.SCNParticleOrientationMode = SCNParticleOrientationMode
exports.SCNSceneSource = SCNSceneSource
exports.SCNNode = SCNNode
exports.SCNSceneExportProgressHandler = SCNSceneExportProgressHandler
exports.SCNPhysicsBallSocketJoint = SCNPhysicsBallSocketJoint
exports.SCNPhysicsBody = SCNPhysicsBody
exports.SCNDebugOptions = SCNDebugOptions
exports.SCNLevelOfDetail = SCNLevelOfDetail
exports.SCNParticleImageSequenceAnimationMode = SCNParticleImageSequenceAnimationMode
exports.SCNMatrix4FromMat4 = SCNMatrix4FromMat4
exports.SCNShadowMode = SCNShadowMode
exports.SCNVector4FromFloat4 = SCNVector4FromFloat4
exports.SCNMaterialProperty = SCNMaterialProperty
exports.SCNNodeRendererDelegate = SCNNodeRendererDelegate
exports.SCNPhysicsSliderJoint = SCNPhysicsSliderJoint
exports.SCNActionRunAction = SCNActionRunAction
exports.SCNBox = SCNBox
exports.SCNParticleEventBlock = SCNParticleEventBlock
exports.SCNTransaction = SCNTransaction
exports.SCNActionRemove = SCNActionRemove
exports.SCNBlendMode = SCNBlendMode
exports.SCNTimingFunction = SCNTimingFunction
exports.SCNCone = SCNCone
exports.SCNScene = SCNScene
exports.SCNPhysicsTurbulenceField = SCNPhysicsTurbulenceField
exports.SCNAction = SCNAction
exports.SCNFloor = SCNFloor
exports.SCNWrapMode = SCNWrapMode
exports.SCNText = SCNText
exports._SCNDefaultPBRFragmentShader = _SCNDefaultPBRFragmentShader
exports.SCNActionWait = SCNActionWait
exports.SCNGeometry = SCNGeometry
exports.SCNParticleBlendMode = SCNParticleBlendMode
exports.SCNActionTimingFunction = SCNActionTimingFunction
exports._SCNMtlLoader = _SCNMtlLoader
exports.SCNMorpherCalculationMode = SCNMorpherCalculationMode
exports.SCNVector3ToFloat3 = SCNVector3ToFloat3
exports.SCNGeometryPrimitiveType = SCNGeometryPrimitiveType
exports.SCNSceneSourceStatusHandler = SCNSceneSourceStatusHandler
exports.SCNPhysicsWorld = SCNPhysicsWorld
exports.SCNVector4EqualToVector4 = SCNVector4EqualToVector4
exports.SCNCapsule = SCNCapsule
exports.SCNSceneRenderer = SCNSceneRenderer
exports.SCNMatrix4MakeTranslation = SCNMatrix4MakeTranslation
exports.SCNAnimationEventBlock = SCNAnimationEventBlock
exports.SCNView = SCNView
exports.SCNMatrix4FromGLKMatrix4 = SCNMatrix4FromGLKMatrix4
exports.SCNMatrix4IsIdentity = SCNMatrix4IsIdentity
exports.SCNMorpher = SCNMorpher
exports.SCNSceneRendererDelegate = SCNSceneRendererDelegate
exports.SCNActionJavaScript = SCNActionJavaScript
exports._SCNDefaultParticleVertexShader = _SCNDefaultParticleVertexShader
exports.SCNAccelerationConstraint = SCNAccelerationConstraint
exports.SCNPyramid = SCNPyramid
exports._SCNDefaultShadowVertexShader = _SCNDefaultShadowVertexShader
exports.SCNActionGroup = SCNActionGroup
exports.SCNParticleSystem = SCNParticleSystem
exports.SCNVector3EqualToVector3 = SCNVector3EqualToVector3
exports.SCNPhysicsField = SCNPhysicsField
exports.NSColorSpace = NSColorSpace
exports.NSCoder = NSCoder
exports.NSMutableData = NSMutableData
exports.NSMutableDictionary = NSMutableDictionary
exports.NSData = NSData
exports.NSURL = NSURL
exports.NSKeyedArchiver = NSKeyedArchiver
exports.NSValue = NSValue
exports.NSArray = NSArray
exports.NSNotification = NSNotification
exports.NSKeyedUnarchiver = NSKeyedUnarchiver
exports.NSDictionary = NSDictionary
exports.NotificationCenter = NotificationCenter
exports.NSMutableArray = NSMutableArray
exports.AVAudioNode = AVAudioNode
exports.AVAudioMixerNode = AVAudioMixerNode
exports.GKAgent2D = GKAgent2D
exports.GKSCNNodeComponent = GKSCNNodeComponent
exports.GKGoal = GKGoal
exports.GKPath = GKPath
exports.GKBehavior = GKBehavior
exports.GKScene = GKScene
exports.GKComponent = GKComponent
exports.GKAgentDelegate = GKAgentDelegate
exports.GKEntity = GKEntity
exports.GKAgent = GKAgent
exports.SKSpriteNode = SKSpriteNode
exports.SKColor = SKColor
exports.SKAction = SKAction
exports.SKActionTimingMode = SKActionTimingMode
exports.SKBlendMode = SKBlendMode
exports.SKScene = SKScene
exports.SKFade = SKFade
exports.SKLabelHorizontalAlignmentMode = SKLabelHorizontalAlignmentMode
exports.SKSceneScaleMode = SKSceneScaleMode
exports.SKNode = SKNode
exports.SKGroup = SKGroup
exports.SKEffectNode = SKEffectNode
exports.SKTexture = SKTexture
exports.SKTextureFilteringMode = SKTextureFilteringMode
exports.SKShapeNode = SKShapeNode
exports.SKSequence = SKSequence
exports.SKRepeat = SKRepeat
exports.SKScale = SKScale
exports.SKLabelNode = SKLabelNode
exports.SKLabelVerticalAlignmentMode = SKLabelVerticalAlignmentMode
exports.SKWait = SKWait
exports.GCExtendedGamepad = GCExtendedGamepad
exports.GCGamepad = GCGamepad
exports.GCControllerDirectionPad = GCControllerDirectionPad
exports.GCControllerElement = GCControllerElement
exports.GCMicroGamepad = GCMicroGamepad
exports.GCControllerAxisInput = GCControllerAxisInput
exports.GCControllerButtonInput = GCControllerButtonInput
exports.GCController = GCController
exports.CAMediaTiming = CAMediaTiming
exports.CAAnimationGroup = CAAnimationGroup
exports.CAMediaTimingFunction = CAMediaTimingFunction
exports.CAPropertyAnimation = CAPropertyAnimation
exports.CAKeyframeAnimation = CAKeyframeAnimation
exports.CAAnimationDelegate = CAAnimationDelegate
exports.CAAnimation = CAAnimation
exports.CACodingProxy = CACodingProxy
exports.CABasicAnimation = CABasicAnimation
exports.LKNSArrayCodingProxy = LKNSArrayCodingProxy
exports.CATransform3D = CATransform3D
exports.CAAction = CAAction
exports.LKNSValueCodingProxy = LKNSValueCodingProxy
exports.CACurrentMediaTime = CACurrentMediaTime
exports.NSColor = NSColor
exports.NSColorSpaceModel = NSColorSpaceModel
exports.CFAbsoluteTimeGetCurrent = CFAbsoluteTimeGetCurrent
exports.NSObject = NSObject

exports.kCAFillModeRemoved = constants.kCAFillModeRemoved
exports.kCAFillModeForwards = constants.kCAFillModeForwards
exports.kCAFillModeBackwards = constants.kCAFillModeBackwards
exports.kCAFillModeBoth = constants.kCAFillModeBoth
exports.kCAMediaTimingFunctionLinear = constants.kCAMediaTimingFunctionLinear
exports.kCAMediaTimingFunctionEaseIn = constants.kCAMediaTimingFunctionEaseIn
exports.kCAMediaTimingFunctionEaseOut = constants.kCAMediaTimingFunctionEaseOut
exports.kCAMediaTimingFunctionEaseInEaseOut = constants.kCAMediaTimingFunctionEaseInEaseOut
exports.kCAMediaTimingFunctionDefault = constants.kCAMediaTimingFunctionDefault
exports.kCAAnimationLinear = constants.kCAAnimationLinear
exports.kCAAnimationDiscrete = constants.kCAAnimationDiscrete
exports.kCAAnimationPaced = constants.kCAAnimationPaced
exports.kCAAnimationCubic = constants.kCAAnimationCubic
exports.kCAAnimationCubicPaced = constants.kCAAnimationCubicPaced
exports.kCAAnimationRotateAuto = constants.kCAAnimationRotateAuto
exports.kCAAnimationRotateAutoReverse = constants.kCAAnimationRotateAutoReverse
